from panda3d.core import loadPrcFileData
loadPrcFileData("", 
"""
win-origin 64 25
win-size 1855 1050
sync-video 0
""")

from direct.showbase.ShowBase import ShowBase
from pandac.PandaModules import *
from direct.actor.Actor import Actor
from direct.directutil.Mopath import Mopath
from direct.interval.IntervalGlobal import *
import sys

from camOb import *

class InspectorGadget(ShowBase):

    def __init__(self):
        self.roboPos=3000
        ShowBase.__init__(self)
        base.cam.node().getDisplayRegion(0).setActive(0)
        base.setFrameRateMeter(True) 
        #self.render.setShaderAuto()
        #render.setAntialias(AntialiasAttrib.MAuto)
        taskMgr.add(self.movement,"moveTask")
        base.disableMouse()

        self.setupMovie()
        self.eventSetup()
        self.camP1 = camOb(0.005, 0.3, 0.505, 0.995)
        self.camP2 = camOb(0.005, 0.3, 0.005, 0.495)
        self.cam1 = self.makeNewDrAndCam('cam1')
        self.cam2 = self.makeNewDrAndCam('cam2')
        self.setupEnv()
        self.setupRobo()
        self.setupMopath()
        self.mopath.goTo(self.robo, self.roboPos)
        self.setupCD()
        self.splitScreen()
        render.analyze()
        
        self.campos = 1
        self.cam1.setPos(self.robo.getX()+700,self.robo.getY(),self.robo.getZ()-75)

        
    def windowAspectEventHandler(self):
        """
        This method is evocked by the eventhnadler when the size of the window changes.
        It curently makes sertain that the relative dimensions of the model in 
        the two 3D renderings are corect and is the same when the aspect ratio changes.
        """
        
        self.cam1.node().getLens().setAspectRatio(self.camP1.getAsp(base.win))
        self.cam2.node().getLens().setAspectRatio(self.camP2.getAsp(base.win))
        
    def eventSetup(self):
        
        self.keyMap = {"swivel-left":0, "swivel-right":0, "forward":0, "humerus-left":0, "humerus-right":0, "ulna-right":0, "ulna-left":0, "backward":0, "toggle":0}
        self.accept('aspectRatioChanged', self.windowAspectEventHandler)
        self.accept("escape", sys.exit)
        self.accept("t", self.eventToggle)
        self.accept("arrow_left", self.setKey, ["swivel-left",1])
        self.accept("arrow_right", self.setKey, ["swivel-right",1])
        self.accept("arrow_up", self.setKey, ["forward",1])
        self.accept("arrow_down", self.setKey, ["backward",1])
        self.accept("a", self.setKey, ["humerus-left",1])
        self.accept("s", self.setKey, ["humerus-right",1])
        self.accept("z", self.setKey, ["ulna-left",1])
        self.accept("x", self.setKey, ["ulna-right",1])
        self.accept("arrow_left-up", self.setKey, ["swivel-left",0])
        self.accept("arrow_right-up", self.setKey, ["swivel-right",0])
        self.accept("arrow_up-up", self.setKey, ["forward",0])
        self.accept("arrow_down-up", self.setKey, ["backward",0])
        self.accept("a-up", self.setKey, ["humerus-left",0])
        self.accept("s-up", self.setKey, ["humerus-right",0])
        self.accept("z-up", self.setKey, ["ulna-left",0])
        self.accept("x-up", self.setKey, ["ulna-right",0])

        
    def eventToggle(self):
        if (self.campos):
            self.cam2.setPos(0, 3500, 400)
            self.cam2.setH(180)
            self.campos = 0
        else :
            self.cam2.setPos(0, -3500, 400)
            self.cam2.setH(0)
            self.campos = 1

    def setupEnv(self):
        """
        This function initialises the environment and the 3D modeles.
        """
        self.nacelle = self.loader.loadModel("models/Nacelle3.egg")
        self.nacelle.reparentTo(render)
        self.nacelle.setPos(0,0,0)
        self.nacelle.flattenLight()
        ambientLight = AmbientLight("ambientLight")
        ambientLight.setColor(Vec4(.3, .3, .3, 1))
        plight = PointLight('plight')
        plight.setColor(VBase4(1, 1, 1, 1))
        plight.setShadowCaster(True, 512, 512)
        plnp = render.attachNewNode(plight)
        plnp.setPos(0, -850, 0)
        render.setLight(plnp)
        
        render.setLight(render.attachNewNode(ambientLight))


    def setupRobo(self):
        self.robo = Actor("models/robo2.egg")
        self.robo.reparentTo(render)
        self.humerus = self.robo.controlJoint(None, 'modelRoot', 'jointFirstArm')
        self.ulna = self.robo.controlJoint(None, 'modelRoot', 'jointSecondArm')
        self.swivel = self.robo.controlJoint(None, 'modelRoot', 'jointSwivel')
        self.Camera = self.robo.controlJoint(None, 'modelRoot', 'jointCamera')
        

    def setupMopath(self):
        self.mopath = Mopath()
        self.mopath.fFaceForward = 1#need to be beffor loadfile
        self.mopath.loadFile("models/Nacelle3.egg")
        print(self.mopath.getMaxT())
        self.mopathLengt=self.mopath.getMaxT()
        
    def setupCD(self):
        base.cTrav = CollisionTraverser()
        base.cTrav.showCollisions(render)
        self.notifier = CollisionHandlerEvent()
        #self.notifier.addInPattern("%fn-in-%in")
        #self.accept("frowney-in-floor", self.onCollision)

    def setupMovie(self):
        """
        This function setsup the video and starts palying it.
        """
        tex = MovieTexture("name")
        assert tex.read("mov/PandaSneezesMjpeg.avi"), "Failed to load video!"
        cm = CardMaker("Panda sneese card");
        cm.setFrame(-0.39, 0.99, -0.2, 0.99)
        cm.setUvRange(tex)
        card = NodePath(cm.generate())
        card.reparentTo(render2d)
        card.setTexture(tex)
        card.setTexScale(TextureStage.getDefault(), tex.getTexScale())
        tex.play()
    

    def makeNewDrAndCam(self, camName):
        dr = base.win.makeDisplayRegion()
        dr.setClearColor(VBase4(0, 0, 0, 1))
        dr.setClearColorActive(True)
        dr.setClearDepthActive(True)
        cam1 = render.attachNewNode(Camera(camName))
        dr.setCamera(cam1)
        cam1.setPos(0, -3500, 400)
        return cam1


    def splitScreen(self):
        dr1 = self.cam1.node().getDisplayRegion(0)
        dr2 = self.cam2.node().getDisplayRegion(0)
        dr1.setDimensions(self.camP1.x1, self.camP1.x2, self.camP1.y1, self.camP1.y2)
        dr2.setDimensions(self.camP2.x1, self.camP2.x2, self.camP2.y1, self.camP2.y2)
        self.cam1.node().getLens().setAspectRatio(self.camP1.getAsp(base.win))
        self.cam2.node().getLens().setAspectRatio(self.camP2.getAsp(base.win))


    def setKey(self, key, value):
        self.keyMap[key] = value


    def movement(self, task):
        self.cam1.lookAt(self.robo)
        if (self.keyMap["humerus-left"]!=0):
            self.humerus.setR(self.humerus.getR() + 75 * globalClock.getDt())
        if (self.keyMap["humerus-right"]!=0):
            self.humerus.setR(self.humerus.getR() - 75 * globalClock.getDt())
        if (self.keyMap["ulna-left"]!=0):
            self.ulna.setR(self.ulna.getR() + 75 * globalClock.getDt())
        if (self.keyMap["ulna-right"]!=0):
            self.ulna.setR(self.ulna.getR() - 75 * globalClock.getDt())
        if (self.keyMap["swivel-right"]!=0):
            self.swivel.setHpr(270,self.swivel.getP() + 75 * globalClock.getDt(),0)
        if (self.keyMap["swivel-left"]!=0):
            self.swivel.setHpr(270,self.swivel.getP() - 75 * globalClock.getDt(),0)

        if (self.keyMap["forward"]!=0):
            self.roboPos=(self.roboPos-50 * globalClock.getDt())%self.mopathLengt
            self.mopath.goTo(self.robo, self.roboPos)
        if (self.keyMap["backward"]!=0):
            self.roboPos=(self.roboPos+50 * globalClock.getDt())%self.mopathLengt
            self.mopath.goTo(self.robo, self.roboPos)
        
               
        camvec = self.robo.getPos() - self.cam1.getPos()
        camdist = camvec.length()
        camvec.normalize()
        
        if (camdist > 400.0):
            res = self.cam1.getPos() + camvec*1
            res.setZ(self.robo.getZ()-75)
            self.cam1.setPos(res)
            camdist = 170.0
        if (camdist < 50.0):
            res = self.cam1.getPos() + camvec*1
            res.setZ(self.robo.getZ()-75)
            self.cam1.setPos(res)
        
        return task.cont


m = InspectorGadget()

run()
